import { world } from '@minecraft/server';
import { Global } from '../Global.js';
import * as Def from '../Definitions/SettingsDefinition.js';
import { SettingsUtil } from '../Utilities.js';
import { AnimationLink } from '../AnimationLink.js';

const SettingsTypes = {
    MultiplayerSupport: "MultiplayerSupport",
    GunBreakBlocks: "GunBreakBlocks",
    ShowPlayerOutlines: "ShowPlayerOutlines",
    ShowSettingsOnEnterWorld: "ShowSettingsOnEnterWorld"
}

class OnChangeSettingsValue {

    static playerOutlines() {
        const active = SettingsUtil.getSettingsValue(SettingsTypes.ShowPlayerOutlines) === 1 ? true : false;
        world.getAllPlayers().forEach(player => {
            player.setDynamicProperty(Global.PlayerDynamicProperties.animation.outlines_active, active);
            AnimationLink.renewClientAnimationVariable(player, Global.PlayerDynamicProperties.animation.outlines_active);
        });
    }

}



const settingsList = {

    //All settings are declared as their default states
    MultiplayerSupport:       new Def.Settings("Multiplayer Support",                SettingsTypes.MultiplayerSupport,       Def.ToggleTypes.Dropdown, true,  function() { return true; },     null, true),
    GunBreakBlocks:           new Def.Settings("Guns Breaking Blocks",               SettingsTypes.GunBreakBlocks,           Def.ToggleTypes.Dropdown, true,  function() { return true; },     null, false),
    ShowPlayerOutlines:       new Def.Settings("Show Player Outlines",               SettingsTypes.ShowPlayerOutlines,       Def.ToggleTypes.Dropdown, true,  function() { return true; },     OnChangeSettingsValue.playerOutlines, false),
    ShowSettingsOnEnterWorld: new Def.Settings("Show Settings After Entering World", SettingsTypes.ShowSettingsOnEnterWorld, Def.ToggleTypes.Dropdown, true,  function() { return true; },     null, false),
    

    [Symbol.iterator]() {
        const entries = Object.entries(this); // Convert object properties to an array
        let index = 0;

        return {
            next: () => {
                while (index < entries.length) {
                    const [name, object] = entries[index++];
                    return { value: { name, object }, done: false };
                }
                return { done: true };
            }
        };
    }
}




world.afterEvents.playerSpawn.subscribe(evetData => {
    if(!evetData.initialSpawn) { return; }
    for(const settings of settingsList) {
        if(settings?.object.onChangeValue != null) {
            settings?.object.onChangeValue();
        }
    }
});

export { SettingsTypes, settingsList, OnChangeSettingsValue };